#![feature(proc_macro_hygiene, decl_macro)]

#[macro_use] extern crate rocket;
#[macro_use] extern crate rocket_contrib;
#[macro_use] extern crate diesel;

use trust_dns_client::client::AsyncClient;
use trust_dns_client::tcp::TcpClientStream;
use trust_dns_proto::xfer::dns_multiplexer::DnsMultiplexer;
use trust_dns_proto::iocompat::AsyncIoTokioAsStd;
use trust_dns_client::rr::dnssec::Signer;
use tokio::net::TcpStream as TokioTcpStream;
use tokio::task;

use std::sync::{Arc, Mutex};

mod models;
mod config;
mod schema;
mod routes;

use routes::users::*;
use routes::zones::*;


#[database("db")]
pub struct DbConn(diesel::SqliteConnection);

type DnsClient = Arc<Mutex<AsyncClient>>;


#[launch]
async fn rocket() -> rocket::Rocket {
    let app_config = config::load("config.toml".into());
    println!("{:#?}", app_config);

    let (stream, handle) = TcpClientStream::<AsyncIoTokioAsStd<TokioTcpStream>>::new(app_config.dns.server);
    let multiplexer = DnsMultiplexer::<_, Signer>::new(stream, handle, None);
    let client = AsyncClient::connect(multiplexer);
    let (client, bg) = client.await.expect("connection failed");
    task::spawn(bg);

    rocket::ignite()
        .manage(Arc::new(Mutex::new(client)))
        .manage(app_config)
        .attach(DbConn::fairing())
        .mount("/api/v1", routes![get_zone_records, create_auth_token, create_user])
}
